"""Private inquisitor API."""
from fastapi import APIRouter, Depends, Response
from grill import Logger
from verifier import VerificationClient

from .auth import LoginResponse, OAuth2RequestForm, login
from .models import Error
from .settings import Settings, get_logger, get_settings, get_verification_client
from .utils import get_source_ip

router = APIRouter(prefix="/private")


@router.post(
    "/auth/token/",
    responses={
        400: {"description": "Incorrect username or password", "model": Error},
        401: {"description": "Insufficient authorization for requested scopes", "model": Error},
    },
)
async def get_access_token(
    response: Response,
    form_data: OAuth2RequestForm = Depends(),
    source_ip: str | None = Depends(get_source_ip),
    settings: Settings = Depends(get_settings),
    verification_client: VerificationClient = Depends(get_verification_client),
    logger: Logger = Depends(get_logger),
) -> LoginResponse:
    """Log in using OAuth2."""
    return await login(
        response=response,
        form_data=form_data,
        source_ip=source_ip,
        settings=settings,
        verification_client=verification_client,
        logger=logger,
    )
