from unittest.mock import call

from fastapi.testclient import TestClient
from freezegun import freeze_time
from verifier import VerificationId

from inquisitor.constants import LogEntryType, LoggerResult, LogReferenceType, Scope
from inquisitor.main import app

from .auth import TestGetAccessToken
from .utils import make_verification

client = TestClient(app)


@freeze_time("2022-01-01T00:00:00Z")
class GetAccessTokenTest(TestGetAccessToken.Base):
    url = "/private/auth/token/"

    def test_login_without_password(self):
        self._test_login({"username": "v:1"}, [Scope.READ])
        self.assertEqual(
            self.verification_client.mock_calls,
            [call.get(VerificationId("v:1"), access_token=None)],
        )
        self.assertLog(
            LogEntryType.LOGIN,
            LoggerResult.SUCCESS,
            input_properties={"username": "v:1", "password": str(False), "scopes": []},
            output_properties={"granted_scopes": [Scope.READ]},
            output_references={
                LogReferenceType.VERIFICATION: "v:1",
                LogReferenceType.CONTACT: "c:1",
            },
        )

    def test_insufficient_permissions(self):
        self.verification_client.get.return_value = make_verification()

        response = client.post(self.url, data={"username": "v:1", "scope": "read write"})

        self.assertEqual(
            response.json(), {"detail": "Insufficient authorization for requested scopes"}
        )
        self.assertEqual(response.status_code, 401)
        self.assertLog(
            LogEntryType.LOGIN,
            LoggerResult.FAIL,
            input_properties={
                "username": "v:1",
                "password": str(False),
                "scopes": [Scope.READ, Scope.WRITE],
            },
            output_properties={"granted_scopes": [Scope.READ]},
            output_references={
                LogReferenceType.VERIFICATION: "v:1",
                LogReferenceType.CONTACT: "c:1",
            },
        )
